////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "arrowtextbox.h"
#include "grafport.h"
#include "lineend.h"

const int ArrowTextBox::ARROWLEN=20;

ArrowTextBox::ArrowTextBox(ShapeView *v, Grafport *g, int x, int y, 
		int w, int h): TextBox(v,g,x,y,w,h) {
	SetLineStyle(LineStyle::SOLID);
	end.SetType(LineEnd::FILLED_ARROW);
}

ArrowTextBox::ArrowTextBox(const NodeShape &c): TextBox(c) {
	SetLineStyle(LineStyle::SOLID);
	end.SetType(LineEnd::FILLED_ARROW);
}

void ArrowTextBox::DrawShape() {
	Point from = Point(GetPosition()->x, GetTopLeft()->y-ARROWLEN);
	Point to = Point(GetPosition()->x, GetTopLeft()->y);
	Point newTo;
	end.Draw(GetGrafport(), &from, &to, &newTo);
	GetGrafport()->DrawLine(&from, &newTo);
}

int ArrowTextBox::GetTopMost() const {
	int t = TextBox::GetTopMost();
	t -= ARROWLEN;
	return t;
}
