////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "shapeview.h"
#include "grafport.h"
#include "box.h"

const int Box::BOX_WIDTH = 76;
const int Box::BOX_HEIGHT = 38;
const int Box::BOX_HANDLES = 8;

Box::Box(ShapeView *v, Grafport *g, int x, int y, int w, int h): 
		NodeShape(v, g, x, y, w, h) {
	SetHandles(BOX_HANDLES);
	Point pt(*GetPosition());
	SetPosition(&pt);
}

Box::Box(const NodeShape &c): NodeShape(c) {
	SetResizable(True);
        SetFixedName(False);
        SetFixedIndexLabel(False);
	SetHandles(BOX_HANDLES);
	int w = (GetWidth() < MIN_WIDTH) ? BOX_WIDTH : GetWidth();
	int h = (GetHeight() < MIN_HEIGHT) ? BOX_HEIGHT : GetHeight();
	SetWidth(w);
	SetHeight(h);
	Point pt(*GetPosition());
	SetPosition(&pt);
}

void Box::DrawOutLine(const Point *p){
	int x = GetView()->Snap(p->x);
	int y = GetView()->Snap(p->y);
	x = x - GetWidth() / 2;
	y = y - GetHeight() / 2;
	SetOutlineAttributes();
	GetGrafport()->DrawRectangle(x, y, GetWidth(), GetHeight());
}

void Box::DrawShape() {
	if (GetFillStyle()==FillStyle::FILLED) {
		GetGrafport()->SetForegroundColor(GetFillColor());
		GetGrafport()->FillRectangle(
			GetTopLeft()->x, GetTopLeft()->y, 
			GetWidth(), GetHeight());
		GetGrafport()->SetForegroundColor(GetColor());
	}
	GetGrafport()->DrawRectangle(
		GetTopLeft()->x, GetTopLeft()->y, 
		GetWidth(), GetHeight());
}

Point Box::GiveSnp(Point to) const {
	return Intersect::BoxIntersects(*GetPosition(), to, 
		GetWidth(), GetHeight());
}

Point Box::GiveOrthoSnp(Point to) const {
	// two special cases for othogonal intersection.
	Point pt;
	Point pos = *GetPosition();
	Point tl = *GetTopLeft();
	if (tl.x < to.x && to.x < tl.x + GetWidth()) {
		pt.x = to.x;
		if (to.y < pos.y)
			pt.y = tl.y;
		else 
			pt.y = tl.y + GetHeight();
		return pt;
	}
	else if (tl.y < to.y && to.y < tl.y + GetHeight()) { 
		pt.y = to.y;
		if (to.x < pos.x)
			pt.x = tl.x;
		else 
			pt.x = tl.x + GetWidth();
		return pt;
	}
	return Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight());
}

Point Box::GiveSnp(Point to, int order, int count, int &code) {
	return Intersect::BoxIntersects(*GetPosition(), to, 
		GetWidth(), GetHeight(), order, count, code);
}
