//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _BOX_H
#define _BOX_H

#include "nodeshape.h"

/// Rectangular shape class.
class Box: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	/// default width
	static const int BOX_WIDTH;
	/// default height
	static const int BOX_HEIGHT;
	///
	Box(ShapeView *v, Grafport *g, int x, int y, int w = BOX_WIDTH, 
		int h = BOX_HEIGHT);
	///
	Box(const NodeShape &);
	///
	Shape *Clone() {return new Box(*this);}

	/// draw box-form dotted outline snaps to the grid.
	void DrawOutLine(const Point *center);

	/// See Shape::
	Point GiveSnp(Point to) const;

	/// See Shape::
	Point GiveOrthoSnp(Point to) const; 	

	/// order and count are for multiple edges. code for horz or vert point.
	Point GiveSnp(Point p, int order, int count, int &code); 

	///
	int GetClassType() const {return Code::BOX;}

	/// default nr. of handles.
	static const int BOX_HANDLES;
protected:
	/// draws a rectangle
	void DrawShape(); 	
};
#endif
