//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam and University of Twente.
// Author: Frank Dehne (frank@cs.vu.nl).
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _BUILDINGBLOCK_H
#define _BUILDINGBLOCK_H

#include "nodeshape.h"

/// building block shape class.
class BuildingBlock: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	///
	static const int BUILDING_BLOCK_WIDTH;
	///
	static const int BUILDING_BLOCK_HEIGHT;
	///
	BuildingBlock(ShapeView *v, Grafport *g, int x, int y, 
		int w = BUILDING_BLOCK_WIDTH, int h = BUILDING_BLOCK_HEIGHT);
	///
	BuildingBlock(const BuildingBlock &);
	///
	BuildingBlock(const NodeShape &);
	///
	virtual ~BuildingBlock();
	///
	Shape *Clone() {return new BuildingBlock(*this);}
	/// draws dotted building block 
	void DrawOutLine(const Point *center); 
	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point p, int order, int count, int &code);
	///
	bool ContainsPt(int x, int y);
	///
	int GetClassType() const {return Code::BUILDING_BLOCK;}
	///
	static const int BUILDING_BLOCK_HANDLES;
protected:
	/// draws a building block
	void DrawShape(); 
	///
	void CalcPoints(int x, int y);
	///
	const Point *GetPoints() const {return points;}
private:
	///
	Point *points;
};
#endif
