////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1998, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "bullseye.h"
#include "grafport.h"

const int BullsEye::CIRCLE_WIDTH = 20;
const int BullsEye::CIRCLE_HEIGHT = 20;
const int BullsEye::DOT_WIDTH = 8;
const int BullsEye::DOT_HEIGHT = 8;

BullsEye::BullsEye(ShapeView *v, Grafport *g, 
		int x, int y, int w, int h): Ellipse(v, g, x, y, w, h) { 
	Init();
}

BullsEye::BullsEye(const NodeShape &c): Ellipse(c) {
	SetSize(CIRCLE_WIDTH, CIRCLE_HEIGHT);
	Init();
}

void BullsEye::Init() {
	string s = "black";
	SetFillColor(&s);
        SetNameString(&string::EMPTY);
	SetResizable(False);
	SetHandles(MIN_HANDLES);
	SetFixedName(True);
	SetFixedIndexLabel(True);
	Point pt(*GetPosition());
        SetPosition(&pt);
}

void BullsEye::DrawShape() {
	Ellipse::DrawShape();
	Point pt = *GetPosition();
        int wd = DOT_WIDTH;
        int ht = DOT_HEIGHT;
	string oldBackground;
        Grafport *g = GetGrafport();
        if (GetFillStyle() != FillStyle::UNFILLED) {
             oldBackground = *g->GetBackgroundColor();
             g->SetBackgroundColor(GetFillColor());
        }
        GetGrafport()->FillEllipse(pt.x-wd/2, pt.y-ht/2, wd, ht);	
	if (!GetGrafport()->DrawsXor())
        	GetGrafport()->DrawEllipse(pt.x-wd/2, pt.y-ht/2, wd, ht);	
        if (GetFillStyle() != FillStyle::UNFILLED)
                g->SetBackgroundColor(&oldBackground);
}
