////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "circle.h"

const int Circle::CIRCLE_WIDTH = 60;

Circle::Circle(ShapeView *v, Grafport *g, int x, int y, int dim): 
	Ellipse(v, g, x, y, dim, dim) {}

Circle::Circle(const NodeShape &c): Ellipse(c) { 
	SetSize(GetWidth(), GetHeight());
}

void Circle::SetSize(int w, int h) {
	// Circle can only be resized to equal width and height.
	// height is resized.
	if (GetWidth() == w) {
		SetWidth(h);
		SetHeight(h);
	}
	// width is resized.
	else if (GetHeight() == h) {
		SetWidth(w);
		SetHeight(w);
	}
	else {
	// both resized
		SetWidth(max(w, h));
		SetHeight(max(w, h));
	}
	// minimum width/height
	if (GetWidth()<MIN_WIDTH || GetHeight()<MIN_HEIGHT) {
		SetWidth(MIN_WIDTH);
		SetHeight(MIN_HEIGHT);
	}
	Ellipse::SetSize(GetWidth(), GetHeight());
}
