////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "shapeview.h"
#include "diamond.h"
#include "grafport.h"

const int Diamond::DIAMOND_WIDTH = 76;
const int Diamond::DIAMOND_HEIGHT = 38;
const int Diamond::DIAMOND_HANDLES = 8;

Diamond::Diamond(ShapeView *v, Grafport *g, int x, int y, int w, int h): 
		NodeShape(v,g,x,y,w,h) {
	SetHandles(DIAMOND_HANDLES);
	Point pt(*GetPosition());
	SetPosition(&pt);
	points = new Point[4];
}

Diamond::Diamond(const Diamond &d): NodeShape(d) {
	points = new Point[4];
	for (int i=0;i<4;i++)
		points[i] = d.points[i];
}

Diamond::Diamond(const NodeShape &d): NodeShape(d) {
	SetResizable(True);
        SetFixedName(False);
        SetFixedIndexLabel(False);
	SetHandles(DIAMOND_HANDLES);
	int w = (GetWidth() < MIN_WIDTH) ? DIAMOND_WIDTH : GetWidth();
        int h = (GetHeight() < MIN_HEIGHT) ? DIAMOND_HEIGHT : GetHeight();
        SetWidth(w);
        SetHeight(h);
	Point pt(*GetPosition());
	SetPosition(&pt);
	points = new Point[4];
}

Diamond::~Diamond() {
	delete[] points;
}

void Diamond::DrawOutLine(const Point *p){
	int x = GetView()->Snap(p->x);
	int y = GetView()->Snap(p->y);
	CalcPoints(x, y);
	SetOutlineAttributes();
	GetGrafport()->DrawPolygon(points, 4);
}

void Diamond::CalcPoints(int x, int y) {
	int a = GetWidth() / 2;
	int b = GetHeight() / 2;
	points[0].x = x;
	points[0].y = y - b;
	points[1].x = x + a;
	points[1].y = y;
	points[2].x = x;
	points[2].y = y + b;
	points[3].x = x - a;
	points[3].y = y;
}

Point Diamond::GiveSnp(Point to) const {
	return Intersect::DiamondIntersects(*GetPosition(), to, 
		GetWidth(), GetHeight());
}

Point Diamond::GiveSnp(Point to, int order, int count, int &code) {
	return Intersect::DiamondIntersects(*GetPosition(), to, 
		GetWidth(), GetHeight(), order, count, code);
}

void Diamond::DrawShape() {
	CalcPoints(GetPosition()->x, GetPosition()->y);
	if (GetFillStyle() == FillStyle::FILLED) {
		GetGrafport()->SetForegroundColor(GetFillColor());
		GetGrafport()->FillPolygon(points, 4);
		GetGrafport()->SetForegroundColor(GetColor());
	}
	GetGrafport()->DrawPolygon(points, 4);
}

bool Diamond::ContainsPt(int a, int b) {
	double w = (double)GetWidth()/2;
	double h = (double)GetHeight()/2;
	double x = a - GetPosition()->x;
	double y = b - GetPosition()->y;
	// inside the 4 lines of the diamond OR
	// inside one of the 4 corner handles.
	return( y <= -(h/w)*x + h &&
		y >= -(h/w)*x - h &&
		y <=  (h/w)*x + h &&
		y >=  (h/w)*x - h) ||
		(HitHandles(a, b) != -1);
}
