//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _DIAMOND_H
#define _DIAMOND_H

#include "nodeshape.h"

/// diamond shape class.
class Diamond: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	///
	static const int DIAMOND_WIDTH;
	///
	static const int DIAMOND_HEIGHT;
	///
	Diamond(ShapeView *v, Grafport *g, int x, int y, 
		int w = DIAMOND_WIDTH, int h = DIAMOND_HEIGHT);
	///
	Diamond(const Diamond &);
	///
	Diamond(const NodeShape &);
	///
	virtual ~Diamond();
	///
	Shape *Clone() {return new Diamond(*this);}
	/// draws dotted diamant snapped to the grid.
	void DrawOutLine(const Point *center); 
	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point p, int order, int count, int &code);
	///
	bool ContainsPt(int x, int y);
	///
	int GetClassType() const {return Code::DIAMOND;}
	///
	static const int DIAMOND_HANDLES;
protected:
	/// draws a diamond
	void DrawShape(); 
	///
	void CalcPoints(int x, int y);
	///
	const Point *GetPoints() const {return points;}
private:
	///
	Point *points;
};
#endif
