//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 2000, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _DISK_H
#define _DISK_H

#include "nodeshape.h"

/// Disk shape class.
class Disk: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	/// default width
	static const int DISK_WIDTH;
	/// default height
	static const int DISK_HEIGHT;
	/// relative size of the ellipse.
	static const double ELLIPSE_FACTOR;
	///
	Disk(ShapeView *v, Grafport *g, int x, int y, int w = DISK_WIDTH, 
		int h = DISK_HEIGHT);
	///
	Disk(const NodeShape &);
	///
	Shape *Clone() {return new Disk(*this);}

	/// draw disk-form dotted outline snaps to the grid.
	void DrawOutLine(const Point *center);

	/// See Shape::
	Point GiveSnp(Point to) const;

	/// See Shape::
	Point GiveOrthoSnp(Point to) const; 	

	/// order and count are for multiple edges. code for horz or vert point.
	Point GiveSnp(Point p, int order, int count, int &code); 

	///
	int GetClassType() const {return Code::DISK;}

	/// default nr. of handles.
	static const int DISK_HANDLES;

protected:
	/// draws a disk
	void DrawShape(); 	
	///
	Point GiveBoxOrthoSnp(Point to) const;

	///
        void SetPosition(const Point *position, bool snap=True);
        ///
        void SetSize(int w, int h);

	///
	void SetTextShape();
	///
	void CalcPositionName();
};
#endif
