//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _ELLIPSE_H
#define _ELLIPSE_H

#include "nodeshape.h"

/// ellipse shape class.
class Ellipse: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	/// default width.
	static const int ELLIPSE_WIDTH;
	/// default height.
	static const int ELLIPSE_HEIGHT;
	///
	Ellipse(ShapeView *v, Grafport *g, int x, int y, 
		int w = ELLIPSE_WIDTH, int h = ELLIPSE_HEIGHT);
	///
	Ellipse(const NodeShape &c);
	///
	Shape *Clone() {return new Ellipse(*this);}

	///
	void DrawOutLine(const Point *center);

	/// the asterisk has a slightly different position.
	void DrawDuplicate();

	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point p, int count, int order, int &code);

	/// size adjustment is a bit different.
	void AdjustSize();
	///
	int GetClassType() const {return Code::ELLIPSE;}

	/// default nr. of handles.
	static const int ELLIPSE_HANDLES;
protected:
	///
	void DrawShape();
};
#endif
