////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "grafport.h"
#include "shapeview.h"
#include "ellipsedbox.h"

const int EllipsedBox::RD_FACTOR = 7;

EllipsedBox::EllipsedBox(ShapeView *v, Grafport *g, int x, int y, int w, int h): 
	Box(v,g,x,y,w,h) { }

EllipsedBox::EllipsedBox(const NodeShape &c): Box(c) {}

void EllipsedBox::DrawShape() {
	if (GetFillStyle()==FillStyle::FILLED) {
		GetGrafport()->SetForegroundColor(GetFillColor()->getstr());
		GetGrafport()->FillEllipsedRectangle(
			GetTopLeft()->x, GetTopLeft()->y, GetWidth(), 
			GetHeight(), GetRadius());
		GetGrafport()->SetForegroundColor(GetColor()->getstr());
	}
	GetGrafport()->DrawEllipsedRectangle(GetTopLeft()->x,
			GetTopLeft()->y, GetWidth(), 
			GetHeight(), GetRadius());
}

int EllipsedBox::GetRadius() const {
//HZ	return GetWidth() / RD_FACTOR;
	return GetHeight() / 2;
}

void EllipsedBox::DrawOutLine(const Point *p){
	int x = GetView()->Snap(p->x) - GetWidth()/2;
	int y = GetView()->Snap(p->y) - GetHeight()/2;
	SetOutlineAttributes();
	GetGrafport()->DrawEllipsedRectangle(x, y, 
			GetWidth(), GetHeight(), GetRadius());
}

void EllipsedBox::SetSize(int w, int h) {
	if (w > h)
		NodeShape::SetSize(w, h);
}

void EllipsedBox::DrawDuplicate() {
	// same as ellipse
        int x = GetPosition()->x - 9*GetWidth()/20;
        int y = GetPosition()->y - 2*GetHeight()/5;
        GetGrafport()->DrawStringCentered(x, y, "*");
}

Point EllipsedBox::GiveSnp(Point to) const {
        Point pos = *GetPosition();
        Point p = Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight());
        if (p.x < pos.x - GetWidth()/2 + GetHeight()/2) {
                // regard ellipsed box as ellipse
		pos.x = pos.x - GetWidth()/2 + GetHeight()/2;
                return Intersect::EllipseIntersects(pos, to, 
				GetHeight(), GetHeight());
	}
	else if (p.x > pos.x + GetWidth()/2 - GetHeight()/2) {
                // regard ellipsed box as ellipse
		pos.x = pos.x + GetWidth()/2 - GetHeight()/2;
                return Intersect::EllipseIntersects(pos, to, 
				GetHeight(), GetHeight());
	}
        else
                return p;
}

Point EllipsedBox::GiveOrthoSnp(Point to) const {
	Point pt = Box::GiveOrthoSnp(to);
	Point pos = *GetPosition();
	if (pt.x >= pos.x - GetWidth()/2 + GetHeight()/2 &&
	    pt.x <= pos.x + GetWidth()/2 - GetHeight()/2) {
		// regard it as a box.
		return pt;
	}
	else {
		// regard it as an ellipse
		return GiveSnp(to);
	}
}

Point EllipsedBox::GiveSnp(Point to, int order, int count, int &code) {
	Point pos = *GetPosition();
        Point p = Intersect::BoxIntersects(pos, to, GetWidth(),
			GetHeight(), order, count, code);
	if (p.x < pos.x - GetWidth()/2 + GetHeight()/2) {
                // regard ellipsed box as ellipse
                pos.x = pos.x - GetWidth()/2 + GetHeight()/2;
                return Intersect::EllipseIntersects(pos, to,
                                GetHeight(), GetHeight(),
				order, count, code);
        }
        else if (p.x > pos.x + GetWidth()/2 - GetHeight()/2) {
                // regard ellipsed box as ellipse
                pos.x = pos.x + GetWidth()/2 - GetHeight()/2;
                return Intersect::EllipseIntersects(pos, to,
                                GetHeight(), GetHeight(),
				order, count, code);
        }
	else
		return p;
}
