//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1996, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
//------------------------------------------------------------------------------
#ifndef _ELLIPSEDBOX_H
#define _ELLIPSEDBOX_H

#include "box.h"

/// box class drawn as a ellipsed rectangle.
class EllipsedBox: public Box {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	///
	EllipsedBox(ShapeView *v, Grafport *g, int x, int y, 
		int w = BOX_WIDTH, int h = BOX_HEIGHT);
	///
	EllipsedBox(const NodeShape &c);
	///
	Shape *Clone() {return new EllipsedBox(*this);}
	///
	int GetClassType() const {return Code::ELLIPSED_BOX;}
	///
	void DrawOutLine(const Point *center);
	///
	void DrawDuplicate();
	/// 
	Point GiveSnp(Point to) const;
	/// 
	Point GiveOrthoSnp(Point to) const; 	
	/// 
	Point GiveSnp(Point p, int order, int count, int &code); 
protected:
	/// Additional check on size
	void SetSize(int w, int h);

	/// draws a ellipsed rectangle.
	void DrawShape();

	/// return radius of rounded corner.
	int GetRadius() const;
private:
	/// radius factor of the rounded corner.
	static const int RD_FACTOR;
};
#endif
