////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam and University of Twente.
// Author: Frank Dehne (frank@cs.vu.nl).
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "grafport.h"
#include "shapeview.h"
#include "folder.h"

Folder::Folder(ShapeView *v, Grafport *g, int x, int y) :
		Hexagon(v, g, x, y) { 
	CalcPositionName();
}


Folder::Folder(const NodeShape &c): Hexagon(c) {
	CalcPositionName();
}

void Folder::DrawOutLine(const Point* p){
	Hexagon::DrawOutLine(p);
	NodeShape::DrawTextShapes();
	GetGrafport()->SetLineStyle(LineStyle::DOTTED);
	GetGrafport()->DrawLine(points[0].x + 1, points[2].y, 
				points[2].x - 1, points[2].y);
}


void Folder::DrawShape() {
	Hexagon::DrawShape();
        string oldBackground; 
	Grafport *g = GetGrafport();
        if (GetFillStyle() != FillStyle::UNFILLED) {
             oldBackground = *g->GetBackgroundColor();
             g->SetBackgroundColor(GetFillColor());
        }
	g->DrawLine(points[0].x + 1, points[2].y, 
			points[2].x - 1, points[2].y);
	if (GetFillStyle() != FillStyle::UNFILLED)
                g->SetBackgroundColor(&oldBackground);
}

void Folder::CalcPoints(int x, int y) {
	int a = GetWidth()/2;
        int b = GetHeight()/2;
        int c = GetWidth()/3;
        int d = GetHeight()/3;
        points[0].x = x - a;
        points[0].y = y - b;
        points[1].x = x - a + c;
        points[1].y = y - b;
        points[2].x = x - a + c;
        points[2].y = y - b + d;
        points[3].x = x + a;
        points[3].y = y - b + d;
        points[4].x = x + a;
        points[4].y = y + b;
        points[5].x = x - a;
        points[5].y = y + b;
}


Point Folder::GiveSnp(Point to) const {
	Point pos = *GetPosition();
	Point p = Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight());
	// check if hit in "empty part".
	if ((p.x > points[1].x && p.y < points[2].y)) {
		// calculate to smaller box part.
		pos.x = (points[3].x + points[0].x) / 2;
		pos.y = (points[3].y + points[4].y) / 2;
		int ht = points[4].y - points[3].y;
		return Intersect::BoxIntersects(pos, to, GetWidth(), ht); 
	}
	else 
		return p;
}

Point Folder::GiveSnp(Point to, int order, int count, int& code) {
	Point pos = *GetPosition();
	Point p = Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight(), 
			order, count, code);
	// check if hit in "empty part".
	if ((p.x > points[1].x && p.y < points[2].y)) {
		// calculate to smaller box part.
                pos.x = (points[3].x + points[0].x) / 2;
                pos.y = (points[3].y + points[4].y) / 2;
                int ht = points[4].y - points[3].y;
                return Intersect::BoxIntersects(pos, to, GetWidth(), ht,
			order, count, code);
	}
	else
		return p;
}
 
void Folder::SetSize(int w, int h) {
        // if (w > h/3)   int c = GetHeight()/3;  see below
	// w = max(w, h);          we expect at least a square box
	w = max(w, h/3 + 2);
        Hexagon::SetSize(w, h);
        CalcPositionName();
}

void Folder::CalcPositionName() {
        Point pt = *GetPosition();
        pt.y += GetHeight()/6;
        GetName()->SetPosition(&pt);
}

void Folder::SetPosition(const Point *pos, bool b) {
        Hexagon::SetPosition(pos, b);
        CalcPositionName();
}
 
void Folder::SetTextShape() {
        Hexagon::SetTextShape();
        CalcPositionName();
}
 
