//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1997, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _HEXAGON_H
#define _HEXAGON_H

#include "nodeshape.h"

/// hexagon shape class.
class Hexagon: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	///
	static const int HEXAGON_WIDTH;
	///
	static const int HEXAGON_HEIGHT;
	///
	Hexagon(ShapeView *v, Grafport *g, int x, int y, 
		int w = HEXAGON_WIDTH, int h = HEXAGON_HEIGHT);
	///
        Hexagon(const Hexagon &s);
	///
        Hexagon(const NodeShape &s);
	///
	virtual ~Hexagon();
	///
	Shape *Clone() {return new Hexagon(*this);}
	/// draws dotted hexagon snapped to the grid.
	void DrawOutLine(const Point *center); 
	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point p, int order, int count, int &code);
	///
	bool ContainsPt(int x, int y);
	///
	int GetClassType() const {return Code::HEXAGON;}
	///
	static const int HEXAGON_HANDLES;
protected:
	/// draws a hexagon
	void DrawShape(); 
	///
	virtual void CalcPoints(int x, int y);
	///
	const Point *GetPoints() const {return points;}
	///
	Point *points;
};
#endif
