////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "grafport.h"
#include "shapeview.h"
#include "horizontalbar.h"

const int HorizontalBar::BAR_WIDTH = 76;
const int HorizontalBar::BAR_HEIGHT = 30;
const int HorizontalBar::BAR_HANDLES = 6;

HorizontalBar::HorizontalBar(ShapeView *v, Grafport *g, 
			int x, int y, int w, int h): Box(v,g,x,y,w,h) {
	SetHandles(BAR_HANDLES);
}

HorizontalBar::HorizontalBar(const NodeShape &c): Box(c) {
	SetHandles(BAR_HANDLES);
}

void HorizontalBar::DrawOutLine(const Point *p){
	int x = GetView()->Snap(p->x);
	int y = GetView()->Snap(p->y);
	x = x - GetWidth() / 2;
	y = y - GetHeight() / 2;
	SetOutlineAttributes();
	GetGrafport()->DrawLine(x, y, x+GetWidth(), y);
	GetGrafport()->DrawLine(x, y+GetHeight(), x+GetWidth(), y+GetHeight());
}

void HorizontalBar::DrawShape() {
	int x = GetTopLeft()->x;
	int y = GetTopLeft()->y;
	int w = GetWidth();
	int h = GetHeight();
	Grafport *g = GetGrafport();
	if (GetFillStyle()==FillStyle::FILLED) {
                g->SetForegroundColor(GetFillColor());
                g->FillRectangle(x, y, w, h);
		if (!g->DrawsXor())
                	g->DrawRectangle(GetTopLeft()->x, GetTopLeft()->y,
				GetWidth(), GetHeight());
                g->SetForegroundColor(GetColor());
        }
	GetGrafport()->DrawLine(x, y, x+w, y);
	GetGrafport()->DrawLine(x, y+h, x+w, y+h);
}
