//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1996, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _MINIARROWELLIPSE_H
#define _MINIARROWELLIPSE_H

#include "ellipse.h"

/// small circle having small arrow and label on top.
class MiniArrowEllipse: public Ellipse {
/*@Doc: {\large {\bf scope:} Diagram} */
public:
	/// 
	static const int ARROWLEN;
	///
	static const int CIRCLE_WIDTH;
	///
	MiniArrowEllipse(ShapeView *v, Grafport *g, int x, int y,
		int w = CIRCLE_WIDTH, int h = CIRCLE_WIDTH);
	///
	MiniArrowEllipse(const MiniArrowEllipse &c);
	///
	MiniArrowEllipse(const NodeShape &c);
	///
	void SetPosition(const Point *p, bool snap=True);
	///
	void SetAlignment(TextAlign::Type alignment);
	///
	void SetFont(XFont *font);
	///
	void SetName(const string *s);
	///
	TextShape *ChooseTextShape(int x, int y);
	/// 
	Shape *Clone() {return new MiniArrowEllipse(*this);}
	///
	int GetClassType() const {return Code::MINI_ARROW_ELLIPSE;}
	///
	void SetTextShape();
	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point to, int order, int count, int &code);
	///
	bool HasString(const string *s, bool sens, bool sub) const;
	///
	bool HasNameString(const string *, bool, bool) const {return False;}
	///
	bool HasString(const string*, bool, bool, List<TextShape *> *);
private:
	///
	LineEnd end;
	///
	void DrawShape();
	///
	void CalcPositionName();
	///
	int textHeight;
	///
	void Init();
	///
	bool HasMinSize() {return True;}
};
#endif
