//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1997, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _NODESHAPE_H
#define _NODESHAPE_H

#include "gshape.h"
#include "fillstyle.h"

/// (abstract) graphical node shape class, superclass of box, ellipse etc.
class NodeShape: public GShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	/// 
	NodeShape(ShapeView *v, Grafport *g, int x, int y, 
		int w=MIN_WIDTH, int h=MIN_HEIGHT); 
	///
	NodeShape(const NodeShape &c);
	///
	virtual ~NodeShape();

	/// update string of label.
	void UpdateIndexLabel(const string *s);
	///
	void SetIndexLabel(const string *s) {indexLabel->SetString(s);}

	/// draw duplicate asterisk.
	void DrawDuplicate();
	
	/// draw multiplicity number.
	void DrawMultiplicity();

	///
	FillStyle::Type GetFillStyle() const {return fillStyle;}
	///
	void SetFillStyle(FillStyle::Type f) {fillStyle=f;}
	///
	void UpdateFillStyle(FillStyle::Type f);
	///
	const string *GetFillColor() const {return &fillColor;}
	///
	void SetFillColor(const string *c) {fillColor=*c;}
	///
	void UpdateFillColor(const string *c);

	/// adjusts also the label to a new place
	void SetPosition(const Point *p, bool snap=True);
 
	/// adjusts also the label to a new place
	void SetSize(int w, int h);

	///
	void SetGrafport(Grafport *g);
	///
	void SetView(ShapeView *v);
	///
	void SetFont(XFont *ft);
	///
	void SetTextColor(const string *color);
 
	/// Returns if (x,y) is in text-area.
	bool InTextArea(int x, int y);

	///
	int HitHandles(int, int);

	///
	TextShape *ChooseTextShape(int x, int y);

	/// update size to make it fit the text shapes.
	void AdjustSize();

	///
	bool SetAssocSubject(AssocList *al);

	/// write it to file.
	void WriteMembers(OutputFile *ofile);

	/// read and parse it from file.
	bool ReadMembers(InputFile *ifile, double format);

	///
	void SetTextShape();

	///
	TextShape *GetIndexLabel() const {return indexLabel;}
	///
	bool IsFixedIndexLabel() {return fixedIndexLabel;}
	///
	void SetFixedIndexLabel(bool b) {fixedIndexLabel=b;}

	///
	void SetResizable(bool b) {resizable=b;}
	///
	bool IsResizable() {return resizable;}

	///
	void ForceSize(int w, int ht) {GShape::SetSize(w, ht);}

	///
	bool HasTextShape(TextShape *t) const;
	///
	bool HasString(const string *s, bool sens, bool sub) const;
	///
	bool HasString(const string*, bool, bool, List<TextShape *> *);
protected:
	/// draw name + index text shape.
	void DrawTextShapes();
 
	///
	virtual void CalcPositionIndexLabel();
private:
	///
	FillStyle::Type fillStyle;
	///
	string fillColor;
	///
	TextShape *indexLabel;
	///
	bool fixedIndexLabel;
	///
	bool resizable;
};
#endif
