////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1996, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "grafport.h"
#include "shapeview.h"
#include "notebox.h"

NoteBox::NoteBox(ShapeView *v, Grafport *g, int x, int y) :
		Hexagon(v, g, x, y) { }


NoteBox::NoteBox(const NodeShape &c): Hexagon(c) {
}

void NoteBox::DrawOutLine(const Point* p){
	Hexagon::DrawOutLine(p);
	NodeShape::DrawTextShapes();
	GetGrafport()->SetLineStyle(LineStyle::DOTTED);
	GetGrafport()->DrawLine(points[1].x, points[1].y, 
				points[3].x, points[3].y);
}

void NoteBox::DrawShape() {
	Hexagon::DrawShape();
	GetGrafport()->DrawLine(points[1].x, points[1].y, 
				points[3].x, points[3].y);
	if (GetFillStyle() == FillStyle::FILLED) {
                GetGrafport()->SetForegroundColor(GetFillColor());
		Point p[3];
		p[0] = points[1];
		p[1] = points[2];
		p[2] = points[3];
                GetGrafport()->FillPolygon(p, 3);
                GetGrafport()->SetForegroundColor(GetColor());
        }
}

void NoteBox::CalcPoints(int x, int y) {
	int a = GetWidth()/2;
        int b = GetHeight()/2;
        int c = GetHeight()/3;
        points[0].x = x - a;
        points[0].y = y - b;
        points[1].x = x + a - c;
        points[1].y = y - b;
        points[2].x = x + a - c;
        points[2].y = y - b + c;
        points[3].x = x + a;
        points[3].y = y - b + c;
        points[4].x = x + a;
        points[4].y = y + b;
        points[5].x = x - a;
        points[5].y = y + b;
}

Point NoteBox::GiveSnp(Point to) const {
        Point pos = *GetPosition();
        Point p = Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight());
        // connect to the diagonal line?
        if ((p.x > points[1].x && p.y < points[3].y)) {
                // define little diamond between points[1], points[2] and points[3].
                int wd = 2 * (points[3].x - points[1].x);
                int ht = 2 * (points[3].y - points[1].y);
                pos = points[2];
                p = Intersect::DiamondIntersects(pos, to, wd, ht);
                // compensation when 'wrong' side of diamond is crossed.
                if (p.x < points[1].x)
                        p = points[1];
                else if (p.y > points[3].y)
                        p = points[3];
        }
        return p;
}

Point NoteBox::GiveSnp(Point to, int order, int count, int& code) {
        Point pos = *GetPosition();
        Point p = Intersect::BoxIntersects(pos, to, GetWidth(), GetHeight(), order, count, code);
        // connect to the diagonal line?
        if ((p.x > points[1].x && p.y < points[3].y)) {
                // define little diamond between points[1], points[2] and points[3].
                int wd = 2 * (points[3].x - points[1].x);
                int ht = 2 * (points[3].y - points[1].y);
                pos = points[2];
                p = Intersect::DiamondIntersects(pos, to, wd, ht, order, count, code);
                // compensation when 'wrong' side of diamond is crossed.
                if (p.x < points[1].x)
                        p = points[1];
                else if (p.y > points[3].y)
                        p = points[3];
        }
        return p;
}

void NoteBox::SetSize(int w, int h) {
        // if (w > h/3)   int c = GetHeight()/3;  see below
	// w = max(w, h);          we expect at least a square box
	w = max(w, h/3 + 2);
        Hexagon::SetSize(w, h);
}

