////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "intersect.h"
#include "grafport.h"
#include "shapeview.h"
#include "roundedbox.h"

const int RoundedBox::RD_FACTOR = 7;

RoundedBox::RoundedBox(ShapeView *v, Grafport *g, int x, int y, int w, int h): 
	Box(v,g,x,y,w,h) { }

RoundedBox::RoundedBox(const NodeShape &c): Box(c) {
}

void RoundedBox::DrawShape() {
	if (GetFillStyle()==FillStyle::FILLED) {
		GetGrafport()->SetForegroundColor(GetFillColor());
		GetGrafport()->FillRoundedRectangle(
			GetTopLeft()->x, GetTopLeft()->y, GetWidth(), 
			GetHeight(), GetRadius());
		GetGrafport()->SetForegroundColor(GetColor());
	}
	GetGrafport()->DrawRoundedRectangle(GetTopLeft()->x,
			GetTopLeft()->y, GetWidth(), 
			GetHeight(), GetRadius());
}

int RoundedBox::GetRadius() const {
        return min(10, GetWidth() / RD_FACTOR);
}

void RoundedBox::DrawOutLine(const Point *p){
	int x = GetView()->Snap(p->x) - GetWidth()/2;
	int y = GetView()->Snap(p->y) - GetHeight()/2;
	SetOutlineAttributes();
	GetGrafport()->DrawRoundedRectangle(x, y, 
			GetWidth(), GetHeight(), GetRadius());
}

Point RoundedBox::GiveSnp(Point to) const {
	Point pt = Intersect::BoxIntersects(*GetPosition(), to, 
		GetWidth(), GetHeight());
	return GiveRectSnp(pt);
}

Point RoundedBox::GiveRectSnp(Point pt) const {
	// Check if one of the round corners is hit.
	// is so, return nearest spot on the straight edge.
	int rd = GetRadius();
	Point tl = *GetTopLeft();
	int x1 = tl.x+rd;
	int x2 = tl.x+GetWidth()-rd;
	int y1 = tl.y+rd;
	int y2 = tl.y+GetHeight()-rd;
	if (pt.x < x1) {
		if (pt.y < y1) {
			if (pt.x == tl.x)
				return Point(pt.x, y1);
			else
				return Point(x1, pt.y);
		}
		else if (pt.y > y2) {
			if (pt.x == tl.x)
				return Point(pt.x, y2);
			else
				return Point(x1, pt.y);
		}
	}
	else if (pt.x > x2) {
		if (pt.y < y1) {
			if (pt.x == tl.x+GetWidth())
				return Point(pt.x, y1);
			else
				return Point(x2, pt.y);
		}
		else if (pt.y > y2) {
			if (pt.x == tl.x+GetWidth())
				return Point(pt.x, y2);
			else
				return Point(x2, pt.y);
		}
	}
	return pt;
}

Point RoundedBox::GiveOrthoSnp(Point to) const {
	Point pt = Box::GiveOrthoSnp(to);
	return GiveRectSnp(pt);
}

Point RoundedBox::GiveSnp(Point to, int order, int count, int &code) {
	Point pt = Box::GiveSnp(to, order, count, code);
	return GiveRectSnp(pt);
}
