////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "grafport.h"
#include "gshape.h"
#include "shapeview.h"
#include "diagram.h"
#include "diagramviewer.h"
#include "drawwindow.h"
#include "sizeshapecmd.h"
#include <stdlib.h>

SizeShapeCmd::SizeShapeCmd(GShape *s, int dir): 
		Command(s->GetView()->GetViewer()->GetDiagram(), 
			s->GetView()->GetViewer()) {
	shape = s;
	constrainX = False;
	constrainY = False;
	direction = dir;
	oldWidth = shape->GetWidth();
	oldHeight = shape->GetHeight();
}

void SizeShapeCmd::TrackMouse(TrackType aPhase,
	Point *anchorPoint, Point *previousPoint, Point *nextPoint) {

	Point pt;
	switch(aPhase) {
		case TRACK_PRESS:
			pt = *shape->GetPosition();
			if (abs(pt.x - ScaleCorrect(anchorPoint->x))<6)
				constrainX = True;
			if (abs(pt.y - ScaleCorrect(anchorPoint->y))<6)
				constrainY = True;
			GetMainWindow()->SetSizeCursor(direction);
			DrawOutline(nextPoint);
			break;
		case TRACK_DRAG:
			DrawOutline(previousPoint);
			DrawOutline(nextPoint);
			break;
		case TRACK_RELEASE:
			DrawOutline(previousPoint);
			GetMainWindow()->SetCursor(MouseCursor::LEFT_PTR);
			break;
		default:
			error("unknown track type\n");
			break;
	}
}
