////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1998, Vrije Universiteit Amsterdam and University of Twente.
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "solidhorizontalbar.h"

const int SolidHorizontalBar::BAR_WIDTH = 76;
const int SolidHorizontalBar::BAR_HEIGHT = 4;
const int SolidHorizontalBar::BAR_HANDLES = 2;


SolidHorizontalBar::SolidHorizontalBar(ShapeView *v, Grafport *g, 
			int x, int y, int w, int h): Box(v,g,x,y,w,h) {
	Init();
}

SolidHorizontalBar::SolidHorizontalBar(const NodeShape &c): Box(c) {
	Init();
}

void SolidHorizontalBar::Init() {
	SetHandles(BAR_HANDLES);
	SetWidth(max(GetWidth(), MIN_WIDTH));
	SetHeight(BAR_HEIGHT);
        SetFixedName(True);
        SetFixedIndexLabel(True);
	SetFillStyle(FillStyle::FILLED);
	SetLineStyle(LineStyle::INVISIBLE);
        string s = "black";
        SetFillColor(&s);
	Point pt(*GetPosition());
        SetPosition(&pt);
}

void SolidHorizontalBar::SetSize(int w, int) {
	SetHandles(BAR_HANDLES);
	SetWidth(max(w, MIN_WIDTH));
	SetHeight(BAR_HEIGHT);
        SetFixedName(True);
        SetFixedIndexLabel(True);
	Point pt = *GetPosition();
	PositiveCoord(&pt);
	Point new_topleft;
	new_topleft.x = pt.x - GetWidth() / 2;
	new_topleft.y = pt.y;
	SetTopLeft(&new_topleft);
	SetPositionHandles();
}

int SolidHorizontalBar::HitHandles(int x, int y) {
	// Shape::HitHandles() returns -1 (none), 1 (ToLeftUp) or 2 (ToRightUp)
	int i = Shape::HitHandles(x,y);
	// (this)::HitHandles() returns -1 (none), 7 (ToLeft) or 8 (ToRight)
	return (i == -1 ? -1 : i+6);
}

Point SolidHorizontalBar::GiveSnp(Point to) const {
	return Box::GiveSnp(to);
}

Point SolidHorizontalBar::GiveSnp(Point to, int order, int count, int &code) {
	return Box::GiveSnp(to, order, count, code);
}

bool SolidHorizontalBar::ReadMembers(InputFile *ifile, double format) {
	if (!Box::ReadMembers(ifile, format))
		return False;
	if (format < 1.26) {	// setting defaults
		SetWidth(max(GetWidth(), MIN_WIDTH));
		SetHeight(BAR_HEIGHT);
		SetFillStyle(FillStyle::FILLED);
		string s = "black";
		SetFillColor(&s);
	}
	return True;
}
