////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1998, Vrije Universiteit Amsterdam and University of Twente.
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
// Author: Frank Dehne (frank@cs.vu.nl).

// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "solidverticalbar.h"

const int SolidVerticalBar::BAR_WIDTH = 4;
const int SolidVerticalBar::BAR_HEIGHT = 38;
const int SolidVerticalBar::BAR_HANDLES = 6;


SolidVerticalBar::SolidVerticalBar(ShapeView *v, Grafport *g, 
			int x, int y, int w, int h): Box(v,g,x,y,w,h) {
	Init();
}

SolidVerticalBar::SolidVerticalBar(const NodeShape &c): Box(c) {
	Init();
}

void SolidVerticalBar::Init() {
        SetHandles(BAR_HANDLES);
	SetWidth(BAR_WIDTH);
        SetHeight(max(GetHeight(), MIN_HEIGHT));
        SetFixedName(True);
        SetFixedIndexLabel(True);
	SetFillStyle(FillStyle::FILLED);
	SetLineStyle(LineStyle::INVISIBLE);
        string s = "black";
        SetFillColor(&s);
	Point pt(*GetPosition());
        SetPosition(&pt);
}

void SolidVerticalBar::SetSize(int, int h) {
	SetHandles(BAR_HANDLES);
	SetWidth(BAR_WIDTH);
	SetHeight(max(h, MIN_HEIGHT));
        SetFixedName(True);
        SetFixedIndexLabel(True);
	Point pt = *GetPosition();
	PositiveCoord(&pt);
	Point new_topleft;
	new_topleft.x = pt.x;
	new_topleft.y = pt.y - GetHeight() / 2;
	SetTopLeft(&new_topleft);
	SetPositionHandles();
}

int SolidVerticalBar::HitHandles(int x, int y) {
	// Shape::HitHandles() returns -1 (none), 1 (ToLeftUp) or 2 (ToRightUp)
	// 				3 (ToRightDown) or 4 (ToLeftDown)
	// 				5 (ToDown) or 6 (ToUp)
	int i = Shape::HitHandles(x,y);
	// (this)::HitHandles() returns -1 (none), 5 (ToDown) or 6 (ToUp)
	// 	3 (ToRightDown), 4 (ToLeftDown), 5 (ToDown) ==> 5 (ToDown)
	// 	1 (ToLeftUp), 2 (ToRightUp), 6 (ToUp)	    ==> 6 (ToUp)
	return (i == -1 ? -1 : ((i>=3 && i<=5) ? 5 : 6) );
}

Point SolidVerticalBar::GiveSnp(Point to) const {
	// return GiveOrthoSnp(to);
	return Box::GiveSnp(to);
}


Point SolidVerticalBar::GiveSnp(Point to, int order, int count, int &code) {
	return Box::GiveSnp(to, order, count, code);
}

bool SolidVerticalBar::ReadMembers(InputFile *ifile, double format) {
	if (!Box::ReadMembers(ifile, format))
		return False;
	if (format < 1.26) {	// setting defaults
		SetWidth(BAR_WIDTH);
		SetHeight(max(GetHeight(), MIN_HEIGHT));
		SetFillStyle(FillStyle::FILLED);
		string s = "black";
		SetFillColor(&s);
	}
	return True;
}
