//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam and University of Twente
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl)
// Author: Frank Dehne (frank@cs.vu.nl)
// 
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _SOLIDVERTICALBAR_H
#define _SOLIDVERTICALBAR_H

#include "box.h"
 
/// A solid vertical bar with fixed width 1
class SolidVerticalBar: public Box {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	///
	static const int BAR_WIDTH;
	///
	static const int BAR_HEIGHT;
	///
	SolidVerticalBar(ShapeView *v, Grafport *g, int x, int y, 
		int w = BAR_WIDTH, int h = BAR_HEIGHT);
	///
        SolidVerticalBar(const NodeShape &c);
	///
	Shape *Clone() {return new SolidVerticalBar(*this);}
	///
	Point GiveSnp(Point to) const;
	/// As GiveSnp but takes into account distribution of multiple lines.
	Point GiveSnp(Point, int, int, int &); 
	///
	bool ReadMembers(InputFile *ifile, double format);
	///
	int GetClassType() const {return Code::SOLID_VERTICAL_BAR;}
	///
	static const int BAR_HANDLES;
protected:
	///
	void SetSize(int w, int h);
	///
	int HitHandles(int x, int y);
private:
	///
	void Init();
};
#endif
