//-----------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam and University of Twente.
// Author: Frank Dehne (frank@cs.vu.nl).
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _STICKMAN_H
#define _STICKMAN_H

#include "nodeshape.h"

/// StickMan shape class.
class StickMan: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public:
	/// default width.
	static const int STICKMAN_WIDTH;
	/// default height.
	static const int STICKMAN_HEIGHT;
	///
	StickMan(ShapeView *v, Grafport *g, int x, int y, 
		int w = STICKMAN_WIDTH, int h = STICKMAN_HEIGHT);
	///
	StickMan(const NodeShape &c);
	///
	Shape *Clone() {return new StickMan(*this);}

	///
	void SetPosition(const Point *pos, bool snap=True);
	///
        void SetSize(int w, int h);
	///
	void SetTextShape();

	///
	void AdjustSize();
	///
	void DrawOutLine(const Point *center);

	/// the asterisk has a slightly different position.
	void DrawDuplicate();

	///
	Point GiveSnp(Point to) const;
	///
	Point GiveSnp(Point p, int count, int order, int &code);

	///
	int GetClassType() const {return Code::STICKMAN;}

	/// default nr. of handles.
	static const int STICKMAN_HANDLES;

protected:
	///
	void CalcPositionLabels();
	///
	void DrawShape();
	///
	void DrawTextShapes();
	///
	void DrawStickMan(int x, int y, int wd, int ht);
	///
	void FillStickMan(int x, int y, int wd, int ht);
private:
        ///
        int RequiredHeight();
	///
	int TextHeight();
};
#endif
