//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _TRIANGLE_H
#define _TRIANGLE_H

#include "nodeshape.h"

/// triangle shape class.
class Triangle: public NodeShape {
/*@Doc: {\large {\bf scope:} diagram} */
public: 
	///
	static const int TRIANGLE_WIDTH;
	///
	static const int TRIANGLE_HEIGHT;
	///
	Triangle(ShapeView *v, Grafport *g, int x, int y, 
		int w = TRIANGLE_WIDTH, int h = TRIANGLE_HEIGHT);
	///
	Triangle(const Triangle &t);
	///
        Triangle(const NodeShape &);
	///
	Shape *Clone() {return new Triangle(*this);}

	/// draws triangle dotted outline. snaps to the grid.
	void DrawOutLine(const Point *center); 
	///
	void DrawDuplicate();

	///
	Point GiveSnp(Point to) const;
	/// same as GiveSnp(Point), i.e. not yet implemented.
	Point GiveSnp(Point p, int order, int count, int &code); 

	///
	void SetSize(int w, int h);
	///
	void SetPosition(const Point *position, bool snap=True);
	///
	bool ContainsPt(int x, int y);
	///
	int GetClassType() const {return Code::TRIANGLE;}
	///
	void SetTextShape();
	///
	static const int TRIANGLE_HANDLES;
protected:
	///
	void CalcPoints(int x, int y);
	///
	const Point *GetPoints() {return &points[0];}
private:
	/// the tree corner points.
	Point points[3]; 
	///
	void CalcPositionName();
	/// draws a triangle
	void DrawShape(); 
};
#endif
