//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam and University of Twente.
// Author: Frank Dehne (frank@cs.vu.nl).
// Author: Henk van de Zandschulp (henkz@cs.utwente.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//-----------------------------------------------------------------------------
#include "lstring.h"
#include "diagramviewer.h"
#include "ucdboundarybox.h"
#include "inputfile.h"
#include "outputfile.h"
#include "subject.h"

const int UCDBoundaryBox::TEXTMARGIN = 4;

UCDBoundaryBox::UCDBoundaryBox(ShapeView *v, Grafport *g, 
		int x, int y, int w, int h): Box(v, g, x, y, w, h) {
	Point pt(*GetPosition());
	SetPosition(&pt);
	CalcPositionLabels();
}

void UCDBoundaryBox::SetPosition(const Point *pos, bool snap) {
	Box::SetPosition(pos, snap);
	CalcPositionLabels();
}

void UCDBoundaryBox::SetSize(int w, int h) {
	Box::SetSize(w, h);
	CalcPositionLabels();
}

void UCDBoundaryBox::AdjustSize() {
        int twd = GetName()->GetWidth();
        if (twd > GetWidth()) {
                UpdateSize(twd+2*TEXTMARGIN, GetHeight());
                GetViewer()->ShapePositionUpdate(this);
	}
	if (IsVisible())
		Undraw();
	CalcPositionLabels();
	if (IsVisible())
		Draw();
}

void UCDBoundaryBox::SetTextShape() {
	Box::SetTextShape();
	CalcPositionLabels();
}

void UCDBoundaryBox::CalcPositionLabels() {
	Point pt(*GetTopLeft());
        int textHeight = GetName()->GetFontHeight();
        int twidth = GetName()->GetWidth();
	pt.x += GetWidth() - twidth/2 - TEXTMARGIN;
	pt.y += textHeight;
        GetName()->SetPosition(&pt);
	// set index label position.
	pt.x = GetPosition()->x + GetWidth()*5/12;
        pt.y = GetPosition()->y - GetHeight()/3;
        GetIndexLabel()->SetPosition(&pt);
}
