////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "gdgraph.h"
#include "genericedge.h"
#include "genericnode.h"
#include "emptynode.h"
#include "miniellipse.h"
#include "blackdot.h"
#include "bullseye.h"
#include "circle.h"
#include "square.h"
#include "notebox.h"
#include "roundedbox.h"
#include "ellipsedbox.h"
#include "triangle.h"
#include "hexagon.h"
#include "horizontalbar.h"
#include "verticalbar.h"
#include "solidhorizontalbar.h"
#include "solidverticalbar.h"
#include "diamond.h"
#include "line.h"
#include "textbox.h"
#include "comment.h"
#include "stickman.h"
#include "folder.h"
#include "disk.h"
#include "buildingblock.h"
#include "cube.h"
#include "subfolder.h"
#include "gddiagram.h"
#include "gdviewer.h"
#include "gdwindow.h"

const int GDDiagram::DOT_WIDTH = 8;
 
GDDiagram::GDDiagram(Config *c, GDWindow *d, GDViewer *v, GDGraph *g): 
		Diagram(c,d,v,g) {
	UpdateNodeType(1);
	UpdateEdgeType(1);
}

Thing *GDDiagram::CreateThing(int classNr) {
	Grafport *g = GetDiagramViewer()->GetGrafport();
	ShapeView *v = GetDiagramViewer()->GetCurView();
	Thing *thing = 0;

	if (classNr == Code::BOX)
		thing = new Box(v, g, 0, 0); 
	else if (classNr == Code::SQUARE)
		thing = new Square(v, g, 0, 0); 
	else if (classNr == Code::CIRCLE)
		thing = new Circle(v, g, 0, 0); 
	else if (classNr == Code::ELLIPSE)
		thing = new Ellipse(v, g, 0, 0); 
	else if (classNr == Code::STICKMAN)
		thing = new StickMan(v, g, 0, 0); 
	else if (classNr == Code::TRIANGLE)
		thing = new Triangle(v, g, 0, 0); 
	else if (classNr == Code::HEXAGON)
		thing = new Hexagon(v, g, 0, 0); 
	else if (classNr == Code::HORIZONTAL_BAR)
		thing = new HorizontalBar(v, g, 0, 0); 
	else if (classNr == Code::VERTICAL_BAR)
		thing = new VerticalBar(v, g, 0, 0); 
	else if (classNr == Code::ROUNDED_BOX)
		thing = new RoundedBox(v, g, 0, 0); 
	else if (classNr == Code::ELLIPSED_BOX)
		thing = new EllipsedBox(v, g, 0, 0); 
	else if (classNr == Code::DIAMOND)
		thing = new Diamond(v, g, 0, 0); 
	else if (classNr == Code::TEXT_BOX)
		thing = new TextBox(v, g, 0, 0); 
	else if (classNr == Code::BLACK_DOT)
		thing = new BlackDot(v, g, 0, 0); 
	else if (classNr == Code::BULLS_EYE)
		thing = new BullsEye(v, g, 0, 0); 
	else if (classNr == Code::SOLID_HORIZONTAL_BAR)
                thing = new SolidHorizontalBar(v, g, 0, 0);
	else if (classNr == Code::SOLID_VERTICAL_BAR)
                thing = new SolidVerticalBar(v, g, 0, 0);
	else if (classNr == Code::NOTE_BOX)
                thing = new NoteBox(v, g, 0, 0);
	else if (classNr == Code::BUILDING_BLOCK)
                 thing = new BuildingBlock(v, g, 0, 0);
	else if (classNr == Code::CUBE)
                 thing = new Cube(v, g, 0, 0);
	else if (classNr == Code::FOLDER)
                 thing = new Folder(v, g, 0, 0);
	else if (classNr == Code::SUB_FOLDER)
                 thing = new SubFolder(v, g, 0, 0);
	else if (classNr == Code::DISK)
                 thing = new Disk(v, g, 0, 0);

	else if (classNr == Code::VIEW)
		thing = new ShapeView(GetDiagramViewer()); 

	else if (classNr == Code::LINE)
		thing = new Line(v, g, 0, 0, 0);
	// old line (arrow) types...
	else if (classNr == Code::ARROW) {
		Line *line = new Line(v, g, 0, 0, 0);
		line->SetEnd2(LineEnd::FILLED_ARROW);
		thing = line;
	}
	else if (classNr == Code::DOUBLE_ARROW) {
		Line *line = new Line(v, g, 0, 0, 0);
		line->SetEnd1(LineEnd::FILLED_ARROW);
		line->SetEnd2(LineEnd::FILLED_ARROW);
		thing = line;
	}
	else if (classNr == Code::DOUBLE_HEADED_ARROW) {
		Line *line = new Line(v, g, 0, 0, 0);
		line->SetEnd2(LineEnd::DOUBLE_FILLED_ARROW);
		thing = line;
	}

	else if (classNr == Code::GENERIC_NODE)
		thing = new GenericNode(GetGraph());
	else if (classNr == Code::EMPTY_NODE)
		thing = new EmptyNode(GetGraph());
	else if (classNr == Code::COMMENT)
		thing = new Comment(GetGraph());

	else if (classNr == Code::GENERIC_EDGE)
		thing = new GenericEdge(GetGraph(), 0, 0);
	else {
		error("%s, line %d: impl error: " "wrong class number %d\n", 
			__FILE__, __LINE__, classNr);
		return 0;
	}
	if (thing != 0 && HasIndexShape(thing->GetClassType()))
		((NodeShape *)thing)->SetFixedIndexLabel(False);

	return thing;
}

Node *GDDiagram::CreateNode(){
	Node *node = 0;
	if (GetNodeType() == Code::GENERIC_NODE)
		node = new GenericNode(GetGraph());
	else if (GetNodeType() == Code::EMPTY_NODE)
		node = new EmptyNode(GetGraph());
	else if (GetNodeType() == Code::COMMENT)
		node = new Comment(GetGraph());
	else
		error("%s, line %d: impl error: "
			"unknown node type\n", __FILE__, __LINE__);
	return node;
}

Edge *GDDiagram::CreateEdge(Subject *node1, Subject *node2){
	// Check possible connections (node-node-edge matrix).
	if (!CheckConnection(node1, node2))
		return 0; 
	Edge *edge = 0;
	if (GetEdgeType() == Code::GENERIC_EDGE)
		edge = new GenericEdge(GetGraph(), node1, node2);
	else 
		error( "%s, line %d: impl error: "
			" unknown edge type\n", __FILE__, __LINE__);
	return edge;
}

NodeShape *GDDiagram::CreateNodeShape(Node *node, int x, int y) {
	NodeShape *shape = 0;
	Grafport *g = GetDiagramViewer()->GetGrafport();
	ShapeView *v = GetDiagramViewer()->GetCurView();
	int t = GetNodeShapeType();
	if (t == Code::BOX)
		shape = new Box(v, g, x, y);
	else if (t == Code::SQUARE)
		shape = new Square(v, g, x, y);
	else if (t == Code::CIRCLE)
		shape = new Circle(v, g, x, y);
	else if (t == Code::ELLIPSE)
		shape = new Ellipse(v, g, x, y);
	else if (t == Code::STICKMAN)
		shape = new StickMan(v, g, x, y);
	else if (t == Code::HEXAGON)
		shape = new Hexagon(v, g, x, y);
	else if (t == Code::HORIZONTAL_BAR)
		shape = new HorizontalBar(v, g, x, y);
	else if (t == Code::VERTICAL_BAR)
		shape = new VerticalBar(v, g, x, y);
	else if (t == Code::TRIANGLE)
		shape = new Triangle(v, g, x, y);
	else if (t == Code::ROUNDED_BOX)
		shape = new RoundedBox(v, g, x, y);
	else if (t == Code::ELLIPSED_BOX)
		shape = new EllipsedBox(v, g, x, y);
	else if (t == Code::DIAMOND)
		shape = new Diamond(v, g, x, y);
	else if (t == Code::NOTE_BOX)
		shape = new NoteBox(v, g, x, y);
	else if (t == Code::BLACK_DOT)
		shape = new BlackDot(v, g, x, y);
	else if (t == Code::BULLS_EYE)
		shape = new BullsEye(v, g, x, y);
	else if (t == Code::TEXT_BOX)
		shape = new TextBox(v, g, x, y);
	else if (t == Code::SOLID_HORIZONTAL_BAR)
                shape = new SolidHorizontalBar(v, g, x, y);
	else if (t == Code::SOLID_VERTICAL_BAR)
                shape = new SolidVerticalBar(v, g, x, y);
	else if (t == Code::BUILDING_BLOCK)
	 	shape = new BuildingBlock(v, g, x, y);
	else if (t == Code::CUBE)
	 	shape = new Cube(v, g, x, y);
	else if (t == Code::FOLDER)
	 	shape = new Folder(v, g, x, y);
	else if (t == Code::SUB_FOLDER)
	 	shape = new SubFolder(v, g, x, y);
	else if (t == Code::DISK)
	 	shape = new Disk(v, g, x, y);
	else {
		error( "%s, line %d: impl error: "
			"node shape type does not exist\n", __FILE__, __LINE__);
	}
	if (check(shape)) {
		if (HasIndexShape(t))
			shape->SetFixedIndexLabel(False);
		shape->SetSubject(node);
		shape->SetTextShape();
	}
	return shape;
}

Line *GDDiagram::CreateLine(
		Edge *edge, GShape *from, GShape *to, List<Point *> *l) {
	Grafport *g = GetDiagramViewer()->GetGrafport();
	ShapeView *v = GetDiagramViewer()->GetCurView();
	Line *line = 0;
	int t = GetLineType();
	if (t == Code::LINE)
		line = new Line(v, g, from, to, l, IsCurve());
	else
		error( "%s, line %d: impl error: "
			"line type does not exist\n", __FILE__, __LINE__);
	if (check(line)) {
		line->SetSubject(edge);
		line->SetTextShape();
		line->SetEnd1(GetLineEnd1());
		line->SetEnd2(GetLineEnd2());
	}
	return line;
}

void GDDiagram::UpdateNodeType(int num) {
	((DiagramWindow *)GetMainWindow())->SetNodeName(num);
	switch (num) {
	case 1: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::BOX);
		break;
	case 2: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::CIRCLE);
		break;
	case 3: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::ROUNDED_BOX);
		break;
	case 4: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::TRIANGLE);
		break;
	case 5: SetNodeType(Code::EMPTY_NODE);
		SetNodeShapeType(Code::BULLS_EYE);
		break;
	case 6: SetNodeType(Code::GENERIC_NODE);
	 	SetNodeShapeType(Code::STICKMAN);
	 	break;
	case 7: SetNodeType(Code::GENERIC_NODE);
                SetNodeShapeType(Code::CUBE);
                break;
	case 8: SetNodeType(Code::EMPTY_NODE);
		SetNodeShapeType(Code::SOLID_VERTICAL_BAR);
		break;
	case 9: SetNodeType(Code::COMMENT);
		SetNodeShapeType(Code::TEXT_BOX);
		break;
	case 10: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::DIAMOND);
		break;
	case 11: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::SQUARE);
		break;
	case 12: SetNodeType(Code::GENERIC_NODE);
	 	SetNodeShapeType(Code::NOTE_BOX);
	 	break;
	case 13: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::HEXAGON);
		break;
	case 14: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::FOLDER);
		break;
	case 15: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::BUILDING_BLOCK);
		break;
	case 16: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::VERTICAL_BAR);
		break;
	case 17: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::ELLIPSE);
		break;
	case 18: SetNodeType(Code::EMPTY_NODE);
		SetNodeShapeType(Code::BLACK_DOT);
		break;
	case 19: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::HORIZONTAL_BAR);
		break;
	case 20: SetNodeType(Code::GENERIC_NODE);
	 	SetNodeShapeType(Code::ELLIPSED_BOX);
	 	break;
	case 21: SetNodeType(Code::EMPTY_NODE);
		SetNodeShapeType(Code::SOLID_HORIZONTAL_BAR);
		break;
	case 22: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::SUB_FOLDER);
		break;
	case 23: SetNodeType(Code::GENERIC_NODE);
		SetNodeShapeType(Code::DISK);
		break;
	default:
		error("%s, line %d: impl error: "
		"unknown node type selected\n", __FILE__,__LINE__);
	}
}

void GDDiagram::UpdateEdgeType(int num) {
	((DiagramWindow *)GetMainWindow())->SetEdgeName(num);
	SetEdgeType(Code::GENERIC_EDGE);
	SetLineType(Code::LINE);
	switch(num) {
	case 1: SetLineEnd1(LineEnd::EMPTY);
		SetLineEnd2(LineEnd::EMPTY);
		break;
	case 2: SetLineEnd1(LineEnd::FILLED_ARROW);
		SetLineEnd2(LineEnd::FILLED_ARROW);
		break;
	case 3: SetLineEnd1(LineEnd::EMPTY);
		SetLineEnd2(LineEnd::FILLED_ARROW);
		break;
	case 4: SetLineEnd1(LineEnd::EMPTY);
                SetLineEnd2(LineEnd::WHITE_TRIANGLE);
		break;
	case 5: SetLineEnd1(LineEnd::EMPTY);
		SetLineEnd2(LineEnd::DOUBLE_FILLED_ARROW);
		break;
	case 6: SetLineEnd1(LineEnd::EMPTY);
		SetLineEnd2(LineEnd::OPEN_ARROW);
		break;
	default:
		error("%s, line %d: impl error: "
			"unknown edge type selected\n", __FILE__,__LINE__);
	}
}

bool GDDiagram::HasIndexNode(int code) {
	return (code == Code::GENERIC_NODE);
}

bool GDDiagram::HasIndexShape(int code) {
	return (code==Code::BOX||code==Code::SQUARE||
		code==Code::ELLIPSE||code==Code::CIRCLE||
		code==Code::ROUNDED_BOX||code==Code::DIAMOND||
		code==Code::TRIANGLE||code==Code::HORIZONTAL_BAR||
		code==Code::VERTICAL_BAR||code==Code::NOTE_BOX||
		code==Code::HEXAGON||code==Code::ELLIPSED_BOX||
		code==Code::FOLDER||code==Code::SUB_FOLDER||
		code==Code::CUBE||code==Code::BUILDING_BLOCK||
		code==Code::STICKMAN||code==Code::DISK);
}
