////////////////////////////////////////////////////////////////////////////////
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////
#include "gdupdatenodeshapetypecmd.h"
#include "square.h"
#include "circle.h"
#include "diamond.h"
#include "notebox.h"
#include "triangle.h"
#include "hexagon.h"
#include "roundedbox.h"
#include "ellipsedbox.h"
#include "horizontalbar.h"
#include "stickman.h"
#include "verticalbar.h"
#include "solidhorizontalbar.h"
#include "solidverticalbar.h"
#include "textbox.h"
#include "miniellipse.h"
#include "blackdot.h"
#include "bullseye.h"
#include "subfolder.h"
#include "disk.h"
#include "cube.h"
#include "buildingblock.h"

GDUpdateNodeShapeTypeCmd::GDUpdateNodeShapeTypeCmd(ShapeView *v, 
		List<int *> *l, int n): UpdateNodeShapeTypeCmd(v, l, n) {
	TransformAll();
}

Thing *GDUpdateNodeShapeTypeCmd::Transform(Thing *thing) {
        NodeShape *oldShape = (NodeShape *)thing;
	Thing *newShape;
        if (toType == Code::BOX)
                newShape = new Box(*oldShape);
        else if (toType == Code::ELLIPSE)
                newShape = new Ellipse(*oldShape);
        else if (toType == Code::SQUARE)
                newShape = new Square(*oldShape);
        else if (toType == Code::CIRCLE)
                newShape = new Circle(*oldShape);
        else if (toType == Code::DIAMOND)
                newShape = new Diamond(*oldShape);
        else if (toType == Code::HEXAGON)
                newShape = new Hexagon(*oldShape);
        else if (toType == Code::TRIANGLE)
                newShape = new Triangle(*oldShape);
        else if (toType == Code::HORIZONTAL_BAR)
                newShape = new HorizontalBar(*oldShape);
        else if (toType == Code::VERTICAL_BAR)
                newShape = new VerticalBar(*oldShape);
        else if (toType == Code::ROUNDED_BOX)
                newShape = new RoundedBox(*oldShape);
        else if (toType == Code::ELLIPSED_BOX)
                newShape = new EllipsedBox(*oldShape);
        else if (toType == Code::SOLID_HORIZONTAL_BAR)
                newShape = new SolidHorizontalBar(*oldShape);
        else if (toType == Code::SOLID_VERTICAL_BAR)
                newShape = new SolidVerticalBar(*oldShape);
        else if (toType == Code::BLACK_DOT)
		newShape = new BlackDot(*oldShape);
        else if (toType == Code::BULLS_EYE)
                newShape = new BullsEye(*oldShape);
        else if (toType == Code::TEXT_BOX)
                newShape = new TextBox(*oldShape);
        else if (toType == Code::NOTE_BOX)
                newShape = new NoteBox(*oldShape);
        else if (toType == Code::STICKMAN)
                newShape = new StickMan(*oldShape);
        else if (toType == Code::CUBE)
                newShape = new Cube(*oldShape);
        else if (toType == Code::FOLDER)
                newShape = new Folder(*oldShape);
        else if (toType == Code::DISK)
                newShape = new Disk(*oldShape);
        else if (toType == Code::SUB_FOLDER)
                newShape = new SubFolder(*oldShape);
        else if (toType == Code::BUILDING_BLOCK)
                newShape = new BuildingBlock(*oldShape);
        else {
                error("%s, %d: cannot convert node shape type %d\n",
                	__FILE__, __LINE__, oldShape->GetClassType());
                return 0;
        }
	int oldType = oldShape->GetClassType();
	int newType = newShape->GetClassType();
	if (SolidShape(oldType) && !SolidShape(newType)) {
		((NodeShape *)newShape)->SetFillStyle(FillStyle::UNFILLED);
		((NodeShape *)newShape)->SetLineStyle(LineStyle::SOLID);
	}
	if (oldType == Code::TEXT_BOX) {
		((NodeShape *)newShape)->SetLineStyle(LineStyle::SOLID);
	}
	return newShape;
}

bool GDUpdateNodeShapeTypeCmd::SolidShape(int type) {
	return (type == Code::BLACK_DOT || type == Code::SOLID_VERTICAL_BAR ||
	        type == Code::SOLID_HORIZONTAL_BAR);
}
