//------------------------------------------------------------------------------
//
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1999, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _FIGGRAFPORT_H 
#define _FIGGRAFPORT_H

#include "util.h"
#include "llist.h"
#include "grafport.h"
#include <stdio.h>

/// Fig Grafport Class
class FigGrafport: public Grafport {
/*@Doc: {\large {\bf scope:} GUI} */
public:
	/// open grafport to filename, When filename = "", stdout is used.
	FigGrafport(const char *fileName);
	///
	virtual ~FigGrafport();

	///
	bool IsSucces() {return succes;}

// Sizes, refreshments etc.
	///
	void ClearArea(int, int, int, int) {}

// Line width.
	///
	void SetLineWidth(unsigned n);

// Colors.
	///
	void SetBackgroundColor(const string *);
	///
	void SetBackgroundColor(const char *);
	///
	void SetForegroundColor(const string *);
	///
	void SetForegroundColor(const char *);
 
// Drawing Rectangles.
	///
	void DrawRectangle(int x, int y, int wd, int ht);
	///
	void FillRectangle(int x, int y, int wd, int ht);

	///
	void DrawRoundedRectangle(int x, int y, int wd, int ht, int rd);
	///
	void FillRoundedRectangle(int x, int y, int wd, int ht, int rd);

	///
	void DrawEllipsedRectangle(int x, int y, int wd, int ht, int rd);
	///
	void FillEllipsedRectangle(int x, int y, int wd, int ht, int rd);

	///
	void DrawDisk(int x, int y, int wd, int ht, int rd);
	///
	void FillDisk(int x, int y, int wd, int ht, int rd);

	///
	void DrawUnzoomedRectangle(int x, int y, int wd, int ht);

// Drawing Strings.
	///
	void DrawStringLeft(int x, int y, const char *str);
	///
	void DrawStringCentered(int x, int y, const char *str);
	///
	void DrawStringRight(int x, int y, const char *str);
	///
	void DrawUnzoomedStringLeft(int x, int y, const char *str);
	///
	void DrawUnzoomedStringCentered(int x, int y, const char *str);
	///
	void DrawStringLeftUnderlined(int x, int y, const char *str);
	///
	void DrawStringCenteredUnderlined(int x, int y, const char *str);
	///
	void DrawStringRightUnderlined(int x, int y, const char *str);

// Drawing Lines.
	///
	void DrawLine(int x1, int y1, int x2, int y2);
	///
	void DrawLine(const Point *p1, const Point *p2) {
		DrawLine(p1->x, p1->y, p2->x, p2->y);}
	///
	void DrawCurve(const Point *p1, const Point *p2, 
			const Point *p3, const Point *p4);
	///
	void DrawUnzoomedLine(int x1, int y1, int x2, int y2);


// Drawing Points.
	///
	void DrawPoint(int x, int y);
	///
	void DrawPoint(const Point *p) {DrawPoint(p->x, p->y);}

// Drawing Ellipses.
	///
	void DrawEllipse(int x, int y, int wd, int ht);
	///
	void FillEllipse(int x, int y, int wd, int ht);

// Drawing Arcs.
	///
	void DrawArc(int x, int y, int wd, int ht, int arc1, int arc2);
	///
	void FillArc(int x, int y, int wd, int ht, int arc1, int arc2);

// Drawing Polygons.
	///
	void DrawPolygon(const Point *points, int n);
	///
	void FillPolygon(const Point *points, int n);

// thingies.
	/// generate Fig header.
	void Header(bool landscape=False);

	/// set grafport font.
	void SetFont(XFont *font);

	///
	void SetPageSize(double x, double y) {width = x; height = y;}
	///
	void SetFactor(double f) {factor = f;}

	///
	void SetTexFonts(bool b) {texFonts=b;}
	///
	bool IsTexFonts() {return texFonts;}

protected:
	///
	static void MakeFigString(string *str);
private:
	/// output file.
	FILE *fd;

	/// Factor to convert X coordinates to Fig coordinates.
	static const double F;

	/// Load built-in Fig colors.
	void InitColors();

	/// Xfig pen color number, set by SetForeground.
	int penColor;
	/// Xfig fill color number, set by FillXXXX.
	int fillColor;

	///
	double width;
	///
	double height;
	///
	double factor;
	///
	bool succes;
	///
	int psFont;
	///
	int texFont;

	///
	bool texFonts;

	///
	void DrawFigPolyLine(List<Point *> *points, 
			int sub_type, int radius = -1);
	///
	void DrawFigRectangle(int x, int y, int wd, int ht, 
			int sub_type, int radius = -1);
	///
	void DrawFigEllipse(int x, int y, int rd_x, int rd_y, int sub_type); 
	///
	void DrawFigText(int x, int y, const char *text, int sub_type); 

	///
	void SetFillColor();
	///
	void SetPSFont();
	///
	void SetTexFont();
	///
	void GetFigStringSize(const char *str, double &ht, double &wd);
};
#endif
