//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _PSGRAFPORT_H 
#define _PSGRAFPORT_H

#include "util.h"
#include "llist.h"
#include "grafport.h"
#include <stdio.h>

/// PostScript(tm) grafport class.
class PSGrafport: public Grafport {
/*@Doc: {\large {\bf scope:} GUI} */
public:
	/// open grafport to filename, When filename = "", stdout is used.
	PSGrafport(const char *fileName);
	///
	virtual ~PSGrafport();

	///
	bool IsSucces() {return succes;}

// Sizes, refreshments etc.
	///
	void ClearArea(int, int, int, int) {}

// Line width.
	///
	void SetLineWidth(unsigned n);

// Colors.
	///
	void SetBackgroundColor(const string *);
	///
	void SetBackgroundColor(const char *);
	///
	void SetForegroundColor(const string *);
	///
	void SetForegroundColor(const char *);
 
// Drawing Rectangles.
	///
	void DrawRectangle(int x, int y, int wd, int ht);
	///
	void FillRectangle(int x, int y, int wd, int ht);

	///
	void DrawRoundedRectangle(int x, int y, int wd, int ht, int rd);
	///
	void FillRoundedRectangle(int x, int y, int wd, int ht, int rd);

	///
        void DrawEllipsedRectangle(int x, int y, int wd, int ht, int rd);
        ///
        void FillEllipsedRectangle(int x, int y, int wd, int ht, int rd);
 
	///
        void DrawDisk(int x, int y, int wd, int ht, int ellipse_ht);
        ///
        void FillDisk(int x, int y, int wd, int ht, int ellipse_ht);

	///
	void DrawUnzoomedRectangle(int x, int y, int wd, int ht);

// Drawing Strings.
	///
	void DrawStringLeft(int x, int y, const char *str);
	///
	void DrawStringCentered(int x, int y, const char *str);
	///
	void DrawStringRight(int x, int y, const char *str);
	///
	void DrawUnzoomedStringLeft(int x, int y, const char *str);
	///
	void DrawUnzoomedStringCentered(int x, int y, const char *str);

// Drawing Underlined Strings.
	///
	void DrawStringLeftUnderlined(int x, int y, const char *str);
	///
	void DrawStringRightUnderlined(int x, int y, const char *str);
	///
	void DrawStringCenteredUnderlined(int x, int y, const char *str);

// Drawing Lines.
	///
	void DrawLine(int x1, int y1, int x2, int y2);
	///
	void DrawLine(const Point *p1, const Point *p2) {
		DrawLine(p1->x, p1->y, p2->x, p2->y);}
	///
	void DrawCurve(const Point *p1, const Point *p2, 
			const Point *p3, const Point *p4);
	///
	void DrawUnzoomedLine(int x1, int y1, int x2, int y2);


// Drawing Points.
	///
	void DrawPoint(int x, int y);
	///
	void DrawPoint(const Point *p) {DrawPoint(p->x, p->y);}

// Drawing Ellipses.
	///
	void DrawEllipse(int x, int y, int wd, int ht);
	///
	void FillEllipse(int x, int y, int wd, int ht);

// Drawing Arcs.
	///
	void DrawArc(int x, int y, int wd, int ht, int arc1, int arc2);
	///
	void FillArc(int x, int y, int wd, int ht, int arc1, int arc2);

// Drawing Polygons.
	///
	void DrawPolygon(const Point *points, int n);
	///
	void FillPolygon(const Point *points, int n);

// Postscript thingies.
	/// generate postscript header.
	void Header(const char *title, const char *creator);

	///
	void BeginDrawProc();
	///
	void EndDrawProc();
	///
	void CallDrawProc();

	/// concat banner from file with name of document.
	void Banner(const char *title);

	///
	void BeginPage(int i, int j, int nx, int tot);

	///
	void EndPage();

	/// generate postscript trailer.
	void Trailer();

	/// set grafport font.
	void SetFont(XFont *font);

	/// header for encapsulated postscript + bounding box
	void EPSHeader(const char *title, const char *creator, 
		int llx, int lly, int urx, int ury);

	///
	void EPSProlog(int y, int dy);
	///
	void SetPageSize(double x, double y) {width = x; height = y;}
	///
	void SetFactor(double f) {factor = f;}
	///
	void SetISOLatin1Encoding(bool b) {isoLatin1Encoding = b;}
	///
	void SetShowColors(bool b) {showColors = b;}

	/// postscript file containing front page for printer.
	static const char BANNER[]; 

	///
	static const double XOFF;
	///
	static const double YOFF;
protected:
	///
	static void MakePSString(string *str);
	///
	static void FixEscapeChar(string *str, char c);
	///
	static void FixNonAscii(string *str);
	///
	void FontProlog();
	///
	void InstallIsoLatin1Font(const char *psFont);
	///
	void PageSetup();
	///
	void DrawPSRectangle(int x, int y, int wd, int ht);
	///
	void DrawPSRoundedRectangle(int x, int y, int wd, int ht, int rd);
	///
        void DrawPSEllipsedRectangle(int x, int y, int wd, int ht, int rd);
	///
        void DrawPSArc(int x, int y, int wd, int ht, int ang1, int ang2);
	///
	void DrawPSEllipse(int x, int y, int wd, int ht);
	///
	void DrawPSDisk(int x, int y, int wd, int ht, int ellipse_ht);
	///
	void DrawPSLine(int x1, int y1, int x2, int y2);
	///
	void DrawPSPolygon(const Point *point, int nrPoints);
private:
	/// postscript output file.
	FILE *fd;
	///
	double width;
	///
	double height;
	///
	double factor;
	///
	bool succes;
	///
	bool isoLatin1Encoding;
	///
	bool showColors;

	/// default PostScript line width.
	static const double DEFAULT_LINE_WIDTH;

	///
	List<string> definedFonts;

	///
	void SetPSDashes();
	///
	void UnsetPSDashes();
};
#endif
