//------------------------------------------------------------------------------
// 
// This file is part of Toolkit for Conceptual Modeling (TCM).
// (c) copyright 1995, Vrije Universiteit Amsterdam.
// Author: Frank Dehne (frank@cs.vu.nl).
//
// TCM is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TCM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TCM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.
//-----------------------------------------------------------------------------
#ifndef _XGRAFPORT_H 
#define _XGRAFPORT_H

#include "grafport.h"
#include "llist.h"
#include <X11/Xlib.h>

typedef unsigned long Pixel;

/// X11 grafport for Xlib drawing.
class XGrafport: public Grafport {
/*@Doc: {\large {\bf scope:} GUI} */
public:
	///
	XGrafport(Display *d, Window win, int wd, int ht);
	///
	virtual ~XGrafport();

	///
        bool DrawsXor() {return True;}

// Sizes, refreshment.
	///
	void ClearArea(int x, int y, int wd, int ht);
	///
	int GetWidth() {return width;}
	///
	int GetHeight() {return height;}

	/// redraw using rdPixmap.
	void Redraw(int fromX, int fromY, int wd, int ht, int toX, int toY);
	///
	void UpdateSize(int wd, int ht);

// Colors.
	///
	void SetBackgroundColor(const string *colorName);
	///
	void SetBackgroundColor(const char *colorName);
	///
	void SetForegroundColor(const string *colorName);
	///
	void SetForegroundColor(const char *colorName);
//
	/// linewidth.
	void SetLineWidth(unsigned n);
// Fonts.
	///
	void SetFont(XFont *font);

// Drawing Rectangles.
	///
	void DrawRectangle(int x, int y, int wd, int ht);
	///
	void FillRectangle(int x, int y, int wd, int ht);

	///
	void DrawRoundedRectangle(int x, int y, int wd, int ht, int rd);
	///
	void FillRoundedRectangle(int x, int y, int wd, int ht, int rd);

	///
        void DrawEllipsedRectangle(int x, int y, int wd, int ht, int rd);
        ///
        void FillEllipsedRectangle(int x, int y, int wd, int ht, int rd);

	///
        void DrawDisk(int x, int y, int wd, int ht, int ellipse_ht);
        ///
        void FillDisk(int x, int y, int wd, int ht, int ellipse_ht);

	/// not zoomed, not printed.
	void DrawEditDashedRectangle(int x, int y, int wt, int ht);
	///
	void DrawUnzoomedRectangle(int x, int y, int wt, int ht);

// Drawing Strings.
	///
	void DrawStringLeft(int x, int y, const char *str);
	///
	void DrawStringRight(int x, int y, const char *str);
	///
	void DrawStringCentered(int x, int y, const char *str);
	///
	void DrawStringLeftUnderlined(int x, int y, const char *str);
	///
	void DrawStringRightUnderlined(int x, int y, const char *str);
	///
	void DrawStringCenteredUnderlined(int x, int y, const char *str);

	/// not zoomed
	void DrawUnzoomedStringLeft(int, int, const char *);
	///
	void DrawUnzoomedStringCentered(int, int, const char *);
 
// Drawing Lines.
	///
	void DrawLine(int x1, int y1, int x2, int y2);
	///
	void DrawLine(const Point *p1, const Point *p2) {
		DrawLine(p1->x, p1->y, p2->x, p2->y);}
	///
	void DrawCurve(const Point *p1, const Point *p2, 
			const Point *p3, const Point *p4);
 
	/// not zoomed. Uses different dot pattern.
	void DrawEditDottedGridLine(int x1, int y1, int x2, int y2);
	/// 
	void DrawEditDottedLine(int x1, int y1, int x2, int y2);
 
	/// not zoomed and not printed
	void DrawEditLine(int x1, int y1, int x2, int y2);
	/// not zoomed and not printed
	void DrawEditPoint(int x1, int y2);

	/// not zoomed and not printed
	void DrawEditLine(const Point *p1, const Point *p2) {
		DrawEditLine(p1->x, p1->y, p2->x, p2->y);}
	/// not zoomed and not printed
	void DrawEditPoint(const Point *p) {DrawEditPoint(p->x, p->y);}

	///
	void DrawUnzoomedLine(int, int, int, int);
 
// Drawing Points
	///
	void DrawPoint(int x, int y);
	///
	void DrawPoint(const Point *p) {DrawPoint(p->x, p->y);}

// Drawing Ellipses
	///
	void DrawEllipse(int x, int y, int wd, int ht);
	///
	void FillEllipse(int x, int y, int wd, int ht);

// Drawing Arcs
	///
	void DrawArc(int x, int y, int wd, int ht, int arc1, int arc2);
	///
	void FillArc(int x, int y, int wd, int ht, int arc1, int arc2);

// Drawing Polygons
	///
	void DrawPolygon(const Point *points, int n);
	///
	void FillPolygon(const Point *points, int n);
 
// Drawing Cursor signs.
	///
	void DrawActiveCaret(int x, int y);
	///
	void DrawDeactiveCaret(int x, int y);

protected:
	///
	void DrawXRectangle(GC gc, int x, int y, int wd, int ht);
	///
	void FillXRectangle(GC gc, int x, int y, int wd, int ht);
	///
	void DrawXRoundedRectangle(GC gc, int x, int y, 
		int wd, int ht, int radius);
	///
	void FillXRoundedRectangle(GC gc, int x, int y, 
		int wd, int ht, int radius);
	///
        void DrawXEllipsedRectangle(GC gc, int x, int y,
                int wd, int ht, int radius);
        ///
        void FillXEllipsedRectangle(GC gc, int x, int y,
                int wd, int ht, int radius);
	///
        void DrawXDisk(GC gc, int x, int y,
                int wd, int ht, int ellipse_ht);
        ///
        void FillXDisk(GC gc, int x, int y,
                int wd, int ht, int ellipse_ht);
	///
	void DrawXLine(GC gc, int x1, int y1, int x2, int y2);
	///
	void DrawXArc(GC gc, int x, int y, int wd, int ht, int arc1, int arc2);
	///
	void FillXArc(GC gc, int x, int y, int wd, int ht, int arc1, int arc2);
	///
	void DrawXPolygon(GC gc, const Point *points, int n);
	///
	void FillXPolygon(GC gc, const Point *points, int n);

	/// X graphics context for xor drawing.
	GC CreateXorGC();
	/// X graphics context for copy drawing.
	GC CreateCopyGC();
	/// X gc for drawing xor dotted patterns.
	GC CreateDotGC();
	/// X gc for drawing xor wide dotted patterns.
        GC CreateWideDotGC();
	/// X gc for drawing xor dashed patterns.
	GC CreateDashGC();

	///
	GC GetCopyGC() {return copyGC;}
	///
	GC GetXORGC() {return xorGC;}
	///
	GC GetDotGC() {return dotGC;}
	///
        GC GetWideDotGC() {return wideDotGC;}
	///
	GC GetDashGC() {return dashGC;}

	///
	void ColorInfo();
	///
	void SetDots(GC gc);
	///
	Pixel GetWhitePixel();
	///
	Pixel GetBlackPixel();
	///
	Pixel GetForegroundPixel();
	///
	int Px(double t);
	///
	int Py(double t);
	///
	void CalcCurvePoints(const Point *p1, const Point *p2, 
				const Point *p3, const Point *p4);

private:
	/// The X window where the drawing area is.
	Window 	window;

	/// The X display.
	Display *display;

	/// The width of the drawing area.
	int width;

	/// The height of the drawing area.
	int height;

	/// pixmap for redrawing on exposure
	Pixmap	rdPixmap;

	/// copy GC (Graphic Context). (see Xlib reference man.) 
	GC copyGC;
	/// solid xor GC.
	GC xorGC;
	/// dotted xor GC
	GC dotGC;
	/// wide dotted xor GC
        GC wideDotGC;
	/// dashed xor GC
	GC dashGC;

	/// number of points for approximating a curve.
	enum {CURVE_POINTS = 21};

	/// X colormap.
	Colormap colormap;

	///
	enum {PATTERNLEN = 8};

	/// dot pattern (for EditText)
	char grayPattern[PATTERNLEN];

	///
	bool whitePixelWarned;
	///
	bool blackPixelWarned;

	// Utils for drawing curves.
	///
	int px0, px1, px2, px3;
	///
	int py0, py1, py2, py3;
	///
	int xx[CURVE_POINTS];
	///
	int yy[CURVE_POINTS];
};
#endif
